@@ -8,12 +8,12 @@ Information and methods pertaining to links and link files.
 Author(s): Alec Delaney (Tekktrik)
 """
 
+import fcntl
 import json
 import os
 import pathlib
 import shutil
-import time
-from typing import Dict, List, Tuple, Union
+from typing import Dict, List, Optional, Tuple, Union
 
 from typer import Exit
 from typing_extensions import TypeAlias
@@ -144,16 +144,21 @@ class CircuitPythonLink:
 
         # Save the object as a JSON file
         with open(save_filepath, mode="w", encoding="utf-8") as linkfile:
+            fcntl.lockf(linkfile, fcntl.LOCK_EX)
             json.dump(link_obj, linkfile, indent=4)
+            fcntl.lockf(linkfile, fcntl.LOCK_UN)
 
         # Return the save filepath
         return pathlib.Path(save_filepath)
 
     @classmethod
-    def load_link_by_filepath(cls, link_filepath: str) -> "CircuitPythonLink":
+    def load_link_by_filepath(cls, link_filepath: str) -> Optional["CircuitPythonLink"]:
         """Create a CircuitPythonLink from a JSON file, by filepath."""
         with open(link_filepath, encoding="utf-8") as linkfile:
-            link_obj = json.load(linkfile)
+            link_contents = linkfile.read()
+            if not link_contents:
+                return None
+            link_obj = json.loads(link_contents)
 
         link = cls(
             name=link_obj["name"],
@@ -237,11 +242,10 @@ class CircuitPythonLink:
 
         # Load the link and repeatedly load while not flagged to stop
         temp_link = self.load_link_by_num(self._link_id)
-        while not temp_link.end_flag:
+        while not temp_link or not temp_link.end_flag:
 
             # Load the link
             temp_link = self.load_link_by_num(self._link_id)
-            time.sleep(0.1)
 
             # Detect new files
             read_files = self.get_files_monitored()
@@ -359,11 +363,15 @@ def get_links_header() -> Tuple[str, ...]:
 
 
 def get_links_list(
-    pattern: str, *, abs_paths: bool = False, name: str = ""
+    pattern: str,
+    *,
+    abs_paths: bool = False,
+    name: str = "",
+    folder: str = LINKS_DIRECTORY,
 ) -> List[_TableRowEntry]:
     """Get the information about links."""
     # Get the paths of all the exiting links
-    link_paths = pathlib.Path(LINKS_DIRECTORY).glob(pattern)
+    link_paths = pathlib.Path(folder).glob(pattern)
 
     # Populate the list of links
     link_infos = []
@@ -371,8 +379,6 @@ def get_links_list(
 
         # Load link and start getting info
         link = CircuitPythonLink.load_link_by_filepath(str(link_path))
-        link_id = link.link_id
-        link_name = link.name
         link_running = not link.stopped
 
         # Attempt to use relative paths if possible
@@ -386,22 +392,19 @@ def get_links_list(
 
         # Get remaining link info
         link_write = link.write_path.resolve()
-        link_recursive = link.recursive
-        link_proc = link.process_id
-        link_base = link.base_dir
 
         # If not specific named link is requested, append it
-        if not name or link_name == name:
+        if not name or link.name == name:
             link_infos.append(
                 (
-                    link_id,
-                    link_name,
+                    link.link_id,
+                    link.name,
                     link_running,
                     link_read,
                     link_write,
-                    link_recursive,
-                    link_proc,
-                    link_base,
+                    link.recursive,
+                    link.process_id,
+                    link.base_dir,
                 )
             )
 
